/*************************************************************************\
* Copyright (c) 2006 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* This software is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
\*************************************************************************/

/**********************************************************************
* Asyn device support using ColdFire MCF5282 I2C interface            *
**********************************************************************/       
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory, and
* Berliner Elektronenspeicherring-Gesellschaft m.b.H. (BESSY).
* asynDriver is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/

/*
 * drvAsynI2C.c,v 1.12 2006/06/19 17:22:49 norume Exp
 */

#include <bsp.h>

#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <math.h>

#include <osiUnistd.h>
#include <cantProceed.h>
#include <errlog.h>
#include <iocsh.h>
#include <epicsAssert.h>
#include <epicsStdioRedirect.h>
#include <epicsStdlib.h>
#include <epicsString.h>
#include <epicsThread.h>
#include <epicsEvent.h>
#include <epicsInterrupt.h>
#include <devLib.h>
#include <osiUnistd.h>
#include <epicsExport.h>
#include <asynDriver.h>
#include <asynOctet.h>

/*
 * I2C interrupt vector
 */
#define I2C_INTC0_VECTOR (64+17)

/*
 * This structure holds the hardware-specific information for a single
 * asyn I2C link.
 */
typedef struct {
    int                 connectFlag;
    int                 address;
    unsigned long       nReceived;
    unsigned long       nWrites;
    unsigned long       nReads;
    unsigned long       nSent;
    unsigned long       nInterrupt;
    unsigned long       nIBBset;
    unsigned long       nIIFset;
    unsigned long       nIIFclear;
    unsigned long       nICFclear;
    unsigned long       nReset;
    epicsEventId        interruptEvent;
    asynInterface       common;
    asynInterface       octet;
} i2cController_t;

/*
 * Report link parameters
 */
static void
drvAsynMCF5282I2CReport(void *drvPvt, FILE *fp, int details)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;

    assert(i2c);
    fprintf(fp, "I2C interface: %sonnected\n", i2c->connectFlag ? "C" : "Disc");
    if (details >= 1) {
        fprintf(fp, "          Write transactions: %lu\n", i2c->nWrites);
        fprintf(fp, "           Read transactions: %lu\n", i2c->nReads);
        fprintf(fp, "                        Sent: %lu\n", i2c->nSent);
        fprintf(fp, "                    Received: %lu\n", i2c->nReceived);
        fprintf(fp, "                  Interrupts: %lu\n", i2c->nInterrupt);
        fprintf(fp, "Waits for I2SR[IBB] to clear: %lu\n", i2c->nIBBset);
        fprintf(fp, "I2SR[ICF] unexpectedly clear: %lu\n", i2c->nICFclear);
        fprintf(fp, "  I2SR[IIF] unexpectedly set: %lu\n", i2c->nIIFset);
        fprintf(fp, "I2SR[IIF] unexpectedly clear: %lu\n", i2c->nIIFclear);
        fprintf(fp, "             Hardware resets: %lu\n", i2c->nReset);
    }
}

/*
 * Interrupt handler
 * Very simple-minded.  Just hands off all the work to the I/O thread.
 */
static void
i2c_interrupt_handler(void *arg)
{
    i2cController_t *i2c = (i2cController_t *)arg;

    MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_IIEN;
    i2c->nInterrupt++;
    epicsEventSignal(i2c->interruptEvent);
}

/*
 * Return hardware to power-up idle state
 */
static void resetHardware(asynUser *pasynUser, i2cController_t *i2c)
{
    char dummy;

    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C reset begin I2CR:%#x I2SR:%#x\n",
                                            MCF5282_I2C_I2CR, MCF5282_I2C_I2SR);
    i2c->nReset++;
    MCF5282_I2C_I2CR = 0;
    epicsThreadSleep(0.001);
    MCF5282_I2C_I2CR = MCF5282_I2C_I2CR_IEN | MCF5282_I2C_I2CR_MSTA;
    epicsThreadSleep(0.001);
    dummy = MCF5282_I2C_I2DR;
    epicsThreadSleep(0.001);
    MCF5282_I2C_I2SR = 0;
    epicsThreadSleep(0.001);
    MCF5282_I2C_I2CR = 0;
    epicsThreadSleep(0.001);
    MCF5282_I2C_I2CR = MCF5282_I2C_I2CR_IEN;
    epicsThreadSleep(0.01);
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C reset done I2CR:%#x I2SR:%#x\n",
                                            MCF5282_I2C_I2CR, MCF5282_I2C_I2SR);
}

/*
 * Await I/O completion
 */
static asynStatus
i2c_await_completion(asynUser *pasynUser, i2cController_t *i2c)
{
    uint8 i2sr;

    if (((i2sr = MCF5282_I2C_I2SR) & MCF5282_I2C_I2SR_IIF) != 0) {
        MCF5282_I2C_I2SR = 0;
    }
    else {
        int interruptKey;
        epicsEventWaitStatus waitStatus;

        interruptKey = epicsInterruptLock();
        MCF5282_I2C_I2CR |= MCF5282_I2C_I2CR_IIEN;
        epicsInterruptUnlock(interruptKey);
        waitStatus = epicsEventWaitWithTimeout(i2c->interruptEvent, pasynUser->timeout);
        if (waitStatus != epicsEventWaitOK) {
            interruptKey = epicsInterruptLock();
            MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_IIEN;
            epicsInterruptUnlock(interruptKey);
            MCF5282_I2C_I2SR = 0;
            MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MSTA;

            /*
             * Soak up any pending event that may have been sent between the
             * time that the epicsEventWaitWithTimeout returned and the time
             * that the interrupts were disabled.
             */
            epicsEventTryWait(i2c->interruptEvent);
            if (waitStatus == epicsEventWaitTimeout) {
                epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                    "drvI2C: Timed out waiting for device %d I/O to complete.",
                                                                i2c->address);
                return asynTimeout;
            }
            epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                        "drvI2C: Error waiting for device %d I/O to complete.",
                                                                i2c->address);
            return asynError;
        }
        i2sr = MCF5282_I2C_I2SR;
        MCF5282_I2C_I2SR = 0;
    }

    /*
     * Verify that bus is in correct state
     */
    if ((i2sr & MCF5282_I2C_I2SR_IIF) == 0) {
        i2c->nIIFclear++;
        MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MSTA;
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
            "drvI2C: I2SR[IIF] unexpectedly clear when communicating with device %d.",
                                                                i2c->address);
        return asynError;
    }
    if (i2sr & MCF5282_I2C_I2SR_IAL) {
        MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MSTA;
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                "drvI2C: Arbitration lost when communicating with device %d.",
                                                                i2c->address);
        return asynError;
    }
    if ((i2sr & MCF5282_I2C_I2SR_IBB) == 0) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                "drvI2C: Bus is not active when communicating with device %d.",
                                                                i2c->address);
        return asynError;
    }
    if ((MCF5282_I2C_I2CR & MCF5282_I2C_I2CR_MSTA) == 0) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                "drvI2C: No longer master when communicating with device %d.",
                                                                i2c->address);
        return asynError;
    }
    /*
     * I don't understand how the ICF bit can be clear after
     * the IIF bit has been set (and thus an interrupt generated)
     * but it appears that the hardware sometimes does this.
     */
    if ((i2sr & MCF5282_I2C_I2SR_ICF) == 0) {
        i2c->nICFclear++;
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
            "drvI2C: I2SR[IIF] set but I2SR[ICF] clear when communicating with device %d.",
                                                                i2c->address);
        epicsThreadSleep(0.001);
        MCF5282_I2C_I2DR; /* Dummy read */
        epicsThreadSleep(0.001);
        return asynError;
    }
    if (((MCF5282_I2C_I2CR & MCF5282_I2C_I2CR_MTX) != 0)
     && ((i2sr & MCF5282_I2C_I2SR_RXAK) != 0)) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                    "drvI2C: No ACK when sending to device %d.", i2c->address);
        return asynError;
    }
    return asynSuccess;
}

/*
 * Send the device address
 */
static asynStatus
i2c_send_address(asynUser *pasynUser, i2cController_t *i2c, int readFlag)
{
    int i;

    if (pasynManager->getAddr(pasynUser,&i2c->address) != asynSuccess)
        return asynError;
    if ((i2c->address < 1)  || (i2c->address > 127)) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                            "drvI2C: Invalid device address");
        return asynError;
    }

    /*
     * Verify that bus is idle
     */
    if ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IBB) != 0) {
        asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C bus is not idle I2CR:%#x I2SR:%#x\n",
                                        MCF5282_I2C_I2CR, MCF5282_I2C_I2SR);
        i2c->nIBBset++;
        epicsThreadSleep(0.01);
        if ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IBB) != 0) {
            resetHardware(pasynUser, i2c);
            if ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IBB) != 0) {
                epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                                    "drvI2C: Bus is not idle");
                return asynError;
            }
        }
    }
    if ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IIF) != 0) {
        i2c->nIIFset++;
        MCF5282_I2C_I2SR = 0;
    }

    /*
     * Set transmit mode
     */
    MCF5282_I2C_I2CR |= MCF5282_I2C_I2CR_MTX;

    /*
     * Set master mode
     */
    MCF5282_I2C_I2CR |= MCF5282_I2C_I2CR_MSTA;

    /*
     * Send the address
     */
    MCF5282_I2C_I2DR = (i2c->address << 1) | (readFlag ? 1 : 0);

    /*
     * Wait until bus is ours
     */
    i = 0;
    while ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IBB) == 0) {
        if (++i == 1000) {
            epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                    "drvI2C: Could not take control of bus");
            return asynError;
        }
    }
    return i2c_await_completion(pasynUser, i2c);
}

/*
 * Set frequency divider register
 */
static void
i2c_set_frequency_divider(int bitRate)
{
    epicsUInt32 clock_speed = bsp_get_CPU_clock_speed();
    int i;
    int trueBitRate;
    static const struct {unsigned short ic; unsigned short divider;} tab[] = {
       {0x20,  20}, {0x21,  22}, {0x22,  24}, {0x23,  26},
       {0x00,  28}, {0x24,  28}, {0x01,  30}, {0x25,  32},
       {0x02,  34}, {0x26,  36}, {0x03,  40}, {0x27,  40},
       {0x04,  44}, {0x05,  48}, {0x28,  48}, {0x06,  56},
       {0x29,  56}, {0x2A,  64}, {0x07,  68}, {0x2B,  72},
       {0x08,  80}, {0x2C,  80}, {0x09,  88}, {0x2D,  96},
       {0x0A, 104}, {0x2E, 112}, {0x0B, 128}, {0x2F, 128},
       {0x0C, 144}, {0x0D, 160}, {0x30, 160}, {0x0E, 192},
       {0x31, 192}, {0x32, 224}, {0x0F, 240}, {0x33, 256},
       {0x10, 288}, {0x11, 320}, {0x34, 320}, {0x12, 384},
       {0x35, 384}, {0x36, 448}, {0x13, 480}, {0x37, 512},
       {0x14, 576}, {0x15, 640}, {0x38, 640}, {0x16, 768},
       {0x39, 768}, {0x3A, 896}, {0x17, 960}, {0x3B,1024},
       {0x18,1152}, {0x19,1280}, {0x3C,1280}, {0x1A,1536},
       {0x3D,1536}, {0x3E,1792}, {0x1B,1920}, {0x3F,2048},
       {0x1C,2304}, {0x1D,2560}, {0x1E,3072}, {0x1F,3840}};
    
    for (i = 0 ; i < (sizeof tab / sizeof tab[0]) - 1 ; i++) {
        if ((clock_speed / tab[i].divider) <= bitRate)
            break;
    }
    trueBitRate = clock_speed / tab[i].divider;
    if (fabs((double)(bitRate - trueBitRate) / bitRate) > 0.01)
        printf("Warning: I2C bit rate (%d) differs from desired rate (%d) by > 1%%\n",
                                                        trueBitRate, bitRate);
    MCF5282_I2C_I2FDR = MCF5282_I2C_I2FDR_IC(tab[i].ic);
}

/*
 * Connect
 */
static asynStatus
drvAsynMCF5282I2CConnect(void *drvPvt, asynUser *pasynUser)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;
    int addr;

    /*
     * Sanity check
     */
    assert(i2c);
    if (pasynManager->getAddr(pasynUser,&addr) != asynSuccess)
        return asynError;
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C connect %d.\n", addr);
    if (addr < 0) {
        if (i2c->connectFlag) {
            epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                              "I2C link already connected!");
            return asynError;
        }
        i2c->connectFlag = 1;
        resetHardware(pasynUser, i2c);
    }
    pasynManager->exceptionConnect(pasynUser);
    return asynSuccess;
}

/*
 * Disconnect
 */
static asynStatus
drvAsynMCF5282I2CDisconnect(void *drvPvt, asynUser *pasynUser)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;
    int addr;

    assert(i2c);
    if (pasynManager->getAddr(pasynUser,&addr) != asynSuccess)
        return asynError;
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C disconnect %d.\n", addr);
    if (addr <= 0) {
        if (i2c->connectFlag) {
            i2c->connectFlag = 0;
        }
        else {
            epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                              "I2C link already disconnected!");
            return asynError;
        }
    }
    pasynManager->exceptionDisconnect(pasynUser);
    return asynSuccess;
}

/*
 * Write to a device
 */
static asynStatus drvAsynMCF5282I2CWriteRaw(void *drvPvt, asynUser *pasynUser,
    const char *data, size_t numchars,size_t *nbytesTransfered)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;
    size_t i;
    asynStatus status;

    i2c->nWrites++;
    *nbytesTransfered = 0;
    assert(i2c);
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C write.\n");
    asynPrintIO(pasynUser, ASYN_TRACEIO_DRIVER, data, numchars,
                                                    "I2C write %d ", numchars);
    if (!i2c->connectFlag) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                                        "I2C disconnected");
        return asynError;
    }
    if ((status = i2c_send_address(pasynUser, i2c, 0)) != asynSuccess) {
        resetHardware(pasynUser, i2c);
        return status;
    }
    for (i = 0 ; i < numchars ; i++) {
        if ((MCF5282_I2C_I2SR & MCF5282_I2C_I2SR_IIF) != 0) {
            i2c->nIIFset++;
            epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                "drvI2C: I2SR[IIF] unexpectedly set when communicating with device %d.",
                                                                    i2c->address);
            return asynError;
        }
        MCF5282_I2C_I2DR = *data++;
        if ((status = i2c_await_completion(pasynUser, i2c)) != asynSuccess) {
            resetHardware(pasynUser, i2c);
            *nbytesTransfered = i;
            return status;
        }
        i2c->nSent++;
    }
    MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MSTA;
    *nbytesTransfered = i;
    return asynSuccess;
}

/*
 * Read from a device
 */
static asynStatus drvAsynMCF5282I2CReadRaw(void *drvPvt, asynUser *pasynUser,
    char *data, size_t maxchars,size_t *nbytesTransfered, int *gotEom)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;
    size_t i;
    asynStatus status;

    i2c->nReads++;
    *nbytesTransfered = 0;
    assert(i2c);
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C read.\n");
    if (!i2c->connectFlag) {
        epicsSnprintf(pasynUser->errorMessage,pasynUser->errorMessageSize,
                                                        "I2C disconnected");
        return asynError;
    }
    if ((status = i2c_send_address(pasynUser, i2c, 1)) != asynSuccess) {
        resetHardware(pasynUser, i2c);
        return status;
    }
    MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MTX;  /* Switch to READ mode */
    if (maxchars == 1)
        MCF5282_I2C_I2CR |= MCF5282_I2C_I2CR_TXAK;    /* Inhibit ACK */
    else
        MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_TXAK;   /* Enable ACK */
    i = MCF5282_I2C_I2DR; /* Dummy read */
    for (i = 0 ; i < maxchars ; i++) {
        if ((status = i2c_await_completion(pasynUser, i2c)) != asynSuccess) {
            resetHardware(pasynUser, i2c);
            *nbytesTransfered = i;
            return status;
        }
        if (i == (maxchars - 2))
            MCF5282_I2C_I2CR |= MCF5282_I2C_I2CR_TXAK;    /* Inhibit ACK */
        else if (i == (maxchars - 1))
            MCF5282_I2C_I2CR &= ~MCF5282_I2C_I2CR_MSTA;
        *data++ = MCF5282_I2C_I2DR;
        i2c->nReceived++;
    }
    *nbytesTransfered = i;
    asynPrintIO(pasynUser, ASYN_TRACEIO_DRIVER, data-i, i, "I2C read %d ", i);
    return asynSuccess;
}

/*
 * Flush pending input
 */
static asynStatus
drvAsynMCF5282I2CFlush(void *drvPvt,asynUser *pasynUser)
{
    i2cController_t *i2c = (i2cController_t *)drvPvt;
    int i;

    assert(i2c);
    asynPrint(pasynUser, ASYN_TRACE_FLOW, "I2C flush\n");
    MCF5282_I2C_I2CR = 0;
    MCF5282_I2C_I2CR = MCF5282_I2C_I2CR_IEN | MCF5282_I2C_I2CR_MSTA;
    i = MCF5282_I2C_I2DR;
    MCF5282_I2C_I2SR = 0;
    MCF5282_I2C_I2CR = 0;
    MCF5282_I2C_I2CR = MCF5282_I2C_I2CR_IEN;
    return asynSuccess;
}

/*
 * Clean up a i2cController
 */
static void
i2cCleanup(i2cController_t *i2c)
{
    if (i2c) {
        i2c->connectFlag = 0;
        free(i2c);
    }
}

/*
 * asynCommon methods
 */
static const struct asynCommon drvAsynMCF5282I2CAsynCommon = {
    drvAsynMCF5282I2CReport,
    drvAsynMCF5282I2CConnect,
    drvAsynMCF5282I2CDisconnect
};

/*
 * asynOctet methods
 */
static struct asynOctet drvAsynMCF5282I2CAsynOctet = {
    0,                          /* write */
    drvAsynMCF5282I2CWriteRaw,  /* writeRaw */
    0,                          /* read */
    drvAsynMCF5282I2CReadRaw,   /* readRaw */
    drvAsynMCF5282I2CFlush,     /* flush */
    0,0,                        /* registerInterruptUser, cancelInterruptUser */
    0,0,                        /* setInputEos, getInputEos */
    0,0                         /* setOutputEos, getOutputEos */
};

/*
 * Configure and register the I2C interface
 */
int
drvAsynMCF5282I2CConfigure(char *portName,
                     unsigned int priority,
                     int speed,
                     int noAutoConnect)
{
    i2cController_t *i2c;
    asynStatus status;

    /*
     * Check arguments
     */
    if (portName == NULL) {
        printf("Port name missing.\n");
        return -1;
    }

    /*
     * Create a driver
     */
    i2c = (i2cController_t *)callocMustSucceed(1, sizeof *i2c, "drvAsynMCF5282I2CConfigure()");
    i2c->interruptEvent = epicsEventMustCreate(epicsEventEmpty);
    i2c->connectFlag = 0;

    /*
     *  Link with higher level routines
     */
    i2c->common.interfaceType = asynCommonType;
    i2c->common.pinterface  = (void *)&drvAsynMCF5282I2CAsynCommon;
    i2c->common.drvPvt = i2c;
    i2c->octet.interfaceType = asynOctetType;
    i2c->octet.pinterface  = (void *)&drvAsynMCF5282I2CAsynOctet;
    i2c->octet.drvPvt = i2c;
    if (pasynManager->registerPort(portName,
                                   ASYN_CANBLOCK | ASYN_MULTIDEVICE,
                                   !noAutoConnect,
                                   priority,
                                   0) != asynSuccess) {
        printf("drvAsynMCF5282I2CConfigure: Can't register myself.\n");
        i2cCleanup(i2c);
        return -1;
    }
    status = pasynManager->registerInterface(portName,&i2c->common);
    if(status != asynSuccess) {
        printf("drvAsynMCF5282I2CConfigure: Can't register common.\n");
        i2cCleanup(i2c);
        return -1;
    }
    status = pasynOctetBase->initialize(portName,&i2c->octet,0,0,0);
    if(status != asynSuccess) {
        printf("drvAsynMCF5282I2CConfigure: Can't register octet.\n");
        i2cCleanup(i2c);
        return -1;
    }

    /*
     * Set up hardware
     *   SCL - PAS0 (Alternate UART 2 pins)
     *   SDA - PAS1
     */
    if (((MCF5282_GPIO_PASPAR & MCF5282_GPIO_PASPAR_PASPA0(3)) ==
                                MCF5282_GPIO_PASPAR_PASPA0(2))
     || ((MCF5282_GPIO_PASPAR & MCF5282_GPIO_PASPAR_PASPA1(3)) ==
                                MCF5282_GPIO_PASPAR_PASPA1(2))) {
        printf("drvAsynMCF5282I2CConfigure: I/O Pins are already used by UART 2");
        i2cCleanup(i2c);
        return -1;
    }
    MCF5282_GPIO_PASPAR |= MCF5282_GPIO_PASPAR_PASPA1(3) |
                           MCF5282_GPIO_PASPAR_PASPA0(3);
    if (speed == 0)
        speed = 100000;
    i2c_set_frequency_divider(speed);
    devConnectInterruptVME(I2C_INTC0_VECTOR, i2c_interrupt_handler, i2c);
    return 0;
}

/*
 * IOC shell command registration
 */
#include <iocsh.h>
static const iocshArg drvAsynMCF5282I2CConfigureArg0 = { "port name",iocshArgString};
static const iocshArg drvAsynMCF5282I2CConfigureArg1 = { "priority",iocshArgInt};
static const iocshArg drvAsynMCF5282I2CConfigureArg2 = { "speed [100000]",iocshArgInt};
static const iocshArg drvAsynMCF5282I2CConfigureArg3 = { "noAutoConnect",iocshArgInt};
static const iocshArg *drvAsynMCF5282I2CConfigureArgs[] = {
    &drvAsynMCF5282I2CConfigureArg0, &drvAsynMCF5282I2CConfigureArg1,
    &drvAsynMCF5282I2CConfigureArg2, &drvAsynMCF5282I2CConfigureArg3};
static const iocshFuncDef drvAsynMCF5282I2CConfigureFuncDef =
              {"drvAsynMCF5282I2CConfigure",4,drvAsynMCF5282I2CConfigureArgs};
static void drvAsynMCF5282I2CConfigureCallFunc(const iocshArgBuf *args)
{
    drvAsynMCF5282I2CConfigure(args[0].sval, args[1].ival,
                               args[2].ival, args[3].ival);
}

/*
 * This routine is called before multitasking has started, so there's
 * no race condition in the test/set of firstTime.
 */
static void
drvAsynMCF5282I2CRegisterCommands(void)
{
    static int firstTime = 1;
    if (firstTime) {
        iocshRegister(&drvAsynMCF5282I2CConfigureFuncDef,drvAsynMCF5282I2CConfigureCallFunc);
        firstTime = 0;
    }
}
epicsExportRegistrar(drvAsynMCF5282I2CRegisterCommands);
